// b-xmodemlights.c originally by David Tharp, modifications by Erik Greenwald
// added automatic reconnect on disconnect.
// all modifications are remarked with c++ style remarks ( // instead of /* */ )
//	-Erik <br0ke@shells.clipboard.com> <egg207s@nic.smsu.edu>
// pls read COPYING (gpl)

	/*--------------------------------------------------*
	 |xmodemlights.c                                1.02|
	 |                                                  |
	 | X Modem Lights display                           |
	 |                                                  |
	 | Graphical display of Linux PPP connection status |
	 |                                                  |
	 | Looks a lot like Windows 95's taskbar modem      |
	 | status indicator.  Coincidence?                  |
	 |                                                  |
	 | Intended for use with AfterStep's Wharf          |
	 |                                                  |
	 | requires Xlib, XPM library, shape extension.     |
	 |                                                  |
	 |david w. Tharp <dwtharp@netpci.com>        17Mar98|
	 *--------------------------------------------------*/

	/***** VERSION HISTORY *****
	 *
	 * 1.0/13Mar98/original version, basic functionality with modem and 
	 *   monitors icons.
	 *
	 * 1.01/16Mar98/minor rewrite of code to improve efficiency and add commenting.
	 *
	 * 1.02/17Mar98/added text icon.
	 *
	 ***************************/


/***** DEFINES *****/
 #define ICONWIDTH 48
 #define ICONHEIGHT 48

#define CONNECTWAIT 0		// choose this if the wait is in the ppp script, which I strongly suggest
//#define CONNECTWAIT 120	// if the ppp script doesn't wait, use this instead
#define CONNECTSCRIPT "/etc/ppp/start-ppp"	// hope this isn't too complex...

/***** INCLUDES *****/

/* X11 stuff */
 #include <X11/Xlib.h>             /* basic C language X interface  */
 #include <X11/Xutil.h>            /* X utility routines            */
 #include <X11/xpm.h>              /* XPM library, to make it purdy */
 #include <X11/Xmu/Drawing.h>      /* drawing routines              */
 #include <X11/extensions/shape.h> /* X11 shape extension           */

/* regular UNIX-y stuff */
 #include <stdio.h>                /* standard I/O routines         */
 #include <stdlib.h>               /* standard C library            */
 #include <unistd.h>               /* POSIX-y stuff, I think        */	// cool, this is where system() lives, too
 #include <string.h>               /* basic string routines         */

/* pixmaps and related info */
 #include "xpm/disconnected.xpm"   /* no connection                 */
 #include "xpm/connected.xpm"      /* connected, no activity        */
 #include "xpm/litup.xpm"          /* connected, both lit up        */
 #include "xpm/lightregions"       /* defines light up areas        */

/***** GLOBAL VARIABLES *****/
 /* the names for these are self-explanatory */
 Display *            display;
 int                  screen, depth;
 Window               window;
 XSetWindowAttributes winattr;
 unsigned long        winmask;
 XSizeHints           hints;
 char                 winname[] = "xmodemlights";
 Pixmap               disconnectedshape,
                      connectedshape, 
                      disconnected, 
                      connected, 
                      litup;
 XGCValues            xgcvals;
 GC                   gc;

/***** FUNCTION PROTOTYPES *****/
 Bool pppup();						// the reconnect call is here, too
 void packetcounts(long int * r, long int * s);


/***** MAIN *****/
                
int main(int argc, char ** argv) {

	Bool connectednow = False, pppstatus, flush;
	XEvent event;
	long int sent, rcvd, lsent, lrcvd;
	int accum;

 /* attempt to connect to the display, exit with error otherwise */
	display = XOpenDisplay("");
	if(display == NULL) {
		fprintf(stderr, "\n%s: %s\n\n", argv[0],
			"Darn it!  Where the heck is that display??");
		exit(1);
	}
	
 /* get some useful info about the display */
	screen = DefaultScreen(display);
	depth = DefaultDepth(display, screen);

 /* create the window, with desired attributes, and name it */
	winmask = CWEventMask;  /* the only attribute is the event mask */
	winattr.event_mask = 
		StructureNotifyMask;
	window = XCreateWindow(display, RootWindow(display, screen),
		0, 0, ICONWIDTH, ICONHEIGHT, 0, depth, InputOutput, 
		CopyFromParent, winmask, &winattr);

	XStoreName(display, window, winname);

 /* define and set WM hints for the window */
	hints.flags = PSize;
	hints.width = ICONWIDTH;
	hints.height = ICONHEIGHT;
	XSetNormalHints(display, window, &hints);

 /* create the pixmaps from the included XPMs */
	#define XCPFD(xpm, img, shp) XpmCreatePixmapFromData(display, window, \
		xpm, img, shp, (XpmAttributes *) NULL);

	XCPFD(disconnected_xpm, &disconnected, &disconnectedshape);
  XCPFD(connected_xpm, &connected, &connectedshape);
  XCPFD(litup_xpm, &litup, NULL);

	#undef XCPFD

 /* create a graphics context for our window */
	xgcvals.graphics_exposures = False; /* we don't want exposes when drawing */
	gc = XCreateGC(display, window, GCGraphicsExposures, &xgcvals);
	
	
 /* start with the disconnected window shape and icon */
	XShapeCombineMask(display, window, ShapeBounding, 0, 0, 
		disconnectedshape, ShapeSet);
  XSetWindowBackgroundPixmap(display, window, disconnected);
  XClearWindow(display, window);
  XFlush(display);
	
 /* time to map the window to the screen! */
	XMapWindow(display, window);

 /* main program loop */

	accum = 20; /* this keeps track of when it's time to check for the link */

	while(1) {
	 /* check for events, and die if there's a DestroyNotify */
		while(XPending(display)) {
			XCheckMaskEvent(display, StructureNotifyMask, &event);
			switch(event.type) {
				case DestroyNotify:
					fprintf(stderr, "\n%s: got a DestroyNotify\n\n", argv[0]);
					exit(1);
			}
		}
		/* check for link every 2 seconds */
		if(++accum >= 20) {
			pppstatus = pppup();
			accum = 1;
		}	
		/* exercise the connection, so the pipe breaks when the window is closed */
		XNoOp(display);
		/* do whatever's necessary depending on link status */
		switch(pppstatus) {
			case False:
			 /* if the link is down, and this is the first we've caught it... */
				if(connectednow) {
					connectednow = False;
					XShapeCombineMask(display, window, ShapeBounding, 0, 0, 
						disconnectedshape, ShapeSet);
  				XSetWindowBackgroundPixmap(display, window, disconnected);
  				XClearWindow(display, window);
  				XFlush(display);
				}
				break;
			case True:
				if(!connectednow) {
				 /* if the link is up, but it wasn't last time */
					connectednow = True;
					XShapeCombineMask(display, window, ShapeBounding, 0, 0, 
						connectedshape, ShapeSet);
  				XSetWindowBackgroundPixmap(display, window, connected);
  				XClearWindow(display, window);
  				XFlush(display);
					packetcounts(&rcvd, &sent);
					flush = False;
				}
			 /* save the current number of packets for reference */
				lsent = sent;
				lrcvd = rcvd;
			 /* get the new packet counts */
				packetcounts(&rcvd, &sent);
			 /* clear the window, and draw the necessary indicators */
				XClearWindow(display, window);
				if(sent != lsent) {
					XCopyArea(display, litup, window, gc, send_x, send_y, 
						send_w, send_h, send_x, send_y);
      		accum = 0;
				}
				if(rcvd != lrcvd) {
					XCopyArea(display, litup, window, gc, receive_x, receive_y, 
						receive_w, receive_h, receive_x, receive_y);
      		accum = 0;
				}
				XFlush(display);
				break;
		}
		/* save some CPU cycles ... */
		usleep(100000);
	}
}
/***** END MAIN *****/


/*
 * pppup()
 * check link on ppp0 (check for /var/run/ppp0.pid
 */
Bool pppup() {
	static FILE * pppfile;

	if((pppfile = fopen("/var/run/ppp0.pid", "r")) != NULL) {
		fclose(pppfile);
		return True;
	} else {
		XShapeCombineMask(display, window, ShapeBounding, 0, 0, disconnectedshape, ShapeSet); // black the display (copied from earlier in the program)
		usleep(1000);
		system(CONNECTSCRIPT);		// fork/shell exec the start script
		sleep(CONNECTWAIT);
		return False;
	}
}


/*
 * packetcounts()
 * look in /proc/net/dev for received/sent packet counts
 */
void packetcounts(long int * r, long int * s) {
	static char str[128], dev[32]; /* temporary storage vars */
	static FILE * procnetdev;
	long int j1, j2, j3, j4, j5, j6, j7, j8, j9; /* junk vars */

	procnetdev = fopen("/proc/net/dev", "r");
 /* search until the ppp0 line is found, and store the line in str */
	while(!strstr(fgets(str, 128, procnetdev), "ppp0")) {}
	fclose(procnetdev);
	
 /* extract the desired entries out of the line */
	sscanf(str, "%s %ld %ld %ld %ld %ld %ld %ld %ld %ld %ld %ld",
		&dev, r, &j1, &j2, &j3, &j4, s, &j5, &j6, &j7, &j8, &j9);
}
